/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"

/* Exported functions ---------------------------------------------------------*/
extern  uint8_t CAN_Send(CAN_TypeDef *CAN,CanTxMsg *Message);

/* Global Variables ---------------------------------------------------------*/
xQueueHandle xQueueCAN2,xQueueCAN1;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/
#define REC_TASK_PRIO   ( tskIDLE_PRIORITY + 2 )
#define SEND_TASK_PRIO    ( tskIDLE_PRIORITY + 1 )
#define polynom 0x09

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
void LED_Init(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Diese Task wird alle 100ms aufgerufen und sendet eine CAN Frame.
  * 		Dabei schaltet diese Task nach jedem senden die Orangene LED ein/aus
  * @param  pvParameters not used
  * @retval None
  */
void PeriodicSend(void * pvParameters)
{
	uint8_t i;
    CanTxMsg Message;
	static portTickType xLastWakeTime;

	xLastWakeTime = xTaskGetTickCount();

	//Hier wird der struct (Message) initialisiert der den Can Frame enth�lt
	//Message.RTR = CAN_RTR_DATA	-> diese Frame eine Data Frame ist und KEINE Remote Frame
	//Message.IDE = CAN_ID_STD		-> diese Frame eine Standart Id verwendet und KEIN Extended Id
	//Message.StdId = 0x007			-> die Id dieser Frame ist  007 HEX
	//Message.DLC = 8				-> diese Frame hat 8 Daten Bytes
	//Message.Data[i] =0x55			-> das i-te Daten Byte hat den Wert 55 HEX

	Message.RTR = CAN_RTR_DATA;
	Message.IDE = CAN_ID_STD;
	Message.StdId = 0x111;
	Message.DLC = 8;

	for(i=0;i<8;i++)
		Message.Data[i] = 0x22;


    for( ;; )
      {

    	/* toggle LED each 100ms */
    	GPIO_ToggleBits(GPIOD, GPIO_Pin_13);


    	/**
    	 *Beschreibung:	Diese Funktion ist die Schnittstelle zum CAN Treiber
    	 *Parameter1:	Der erste Parameter legt den CAN Knoten fest welcher die Frame
		 * 				versenden soll. Dies kann sein CAN1 oder CAN2
		 *Parameter2:	Der zweite Parameter nimmt die speicheraddresse des struct auf
		 *				der versendet werden soll
		 *R�ckgabe:		Es wird die Nummer der verwendeten Mailbox zur�ckgegeben
		 *				(in diesem Beispiel wird diese nicht verwendet)
    	  */
    	CAN_Send(CAN1,&Message);

    	/*Wait for 100ms since LastWakeTime*/
    	vTaskDelayUntil( &xLastWakeTime, 100 );
      }


}
/**
  * @brief  Diese Task wird immer aufgerufen wenn eine CAN Frame von CAN1 empfangen wurde.
  * 		Als reaktion darauf wird die rote LED ein/ausgeschaltet
  * @param  pvParameters not used
  * @retval None
  */
void CAN1_MessageReceived ( void * pvParameters )
{
	uint8_t i;
    CanRxMsg RxMessage;

    for(;;)
    {
    	/*Diese Funktion wartet entnimmt ein Element aus der Queue und schreibt dieses
    	 * in die Variable RxMessage. Wenn die Queue leer ist wartet die Funktion und
    	 * blockt die Task. W�hrend diese Task geblockt ist k�nnen andere Tasks
    	 * ausgef�hrt werden bis wieder ein Element in der Queue vorhanden ist.*/
    	xQueueReceive( xQueueCAN1, &( RxMessage ), portMAX_DELAY );

    	/*Toggle LED */
    	GPIO_ToggleBits(GPIOD, GPIO_Pin_14);

    }

}

/**
  * @brief  Diese Task wird immer aufgerufen wenn eine CAN Frame von CAN2 empfangen wurde.
  * 		Als reaktion darauf wird die gr�ne LED ein/ausgeschaltet. Au�erdem wird die ID
  * 		der empfangenen Frame ge�ndert und dann wieder versendet
  * @param  pvParameters not used
  * @retval None
  */
void CAN2_MessageReceived ( void * pvParameters )
{
	uint8_t i;
    CanRxMsg RxMessage;



    for(;;)
    {

    	/*Diese Funktion wartet, entnimmt ein Element aus der Queue und schreibt dieses
    	 * in die Variable RxMessage. Wenn die Queue leer ist wartet die Funktion und
    	 * blockt die Task. W�hrend diese Task geblockt ist k�nnen andere Tasks
    	 * ausgef�hrt werden bis wieder ein Element in der Queue vorhanden ist.*/
    	xQueueReceive( xQueueCAN2, &( RxMessage ), portMAX_DELAY );

    	/*Hier wird die Id der CAN Frame ge�ndert, um sie von der empfangenen Frame
    	 * unterscheiden zu k�nnen*/
    	RxMessage.StdId=0x088;

    	/*Send the Message via CAN2*/
    	CAN_Send(CAN2,&RxMessage);

    	/*Toggle LED*/
    	GPIO_ToggleBits(GPIOD, GPIO_Pin_12);

    }

}


/**
  * @brief  Main program anfangs werden alle Peripherien initialisiert
  * 		danach werden die Tasks erzeugt. Kurz bevor das Betriebssystem
  * 		gestartet wird, werden noch die CAN interrupts scharf geschaltet.
  * @param  None
  * @retval None
  */
int main(void)
{
	SystemInit();
	portBASE_TYPE periodicSend = 0;
	portBASE_TYPE can1MessageReceived = 0;
	portBASE_TYPE can2MessageReceived = 0;
	int8_t checkTasks = 0;
	int8_t numTasks = 3;
	int32_t pauseValue;
	static portTickType xLastWakeTime;
	/*!< At this stage the microcontroller clock setting is already configured to
	   168 MHz, this is done through SystemInit() function which is called from
	   startup file (startup_stm32f4xx.s) before to branch to application main.
	   To reconfigure the default setting of SystemInit() function, refer to
	   system_stm32f4xx.c file
	 */

	/*Initialize Leds */
	LED_Init();

	/*Configure the Interrupt*/
	InterruptCAN_Init();

	/*Diese Funktion initialisiert die CAN Module mit Bitrate und verwendete Pins*/
	CAN_Initialisierung();



	/*Create the Queues who are used in the Tasks*/
	xQueueCAN2 = xQueueCreate( 1, sizeof( CanRxMsg ) );
	xQueueCAN1 = xQueueCreate( 1, sizeof( CanRxMsg ) );

	/*was the Queue successful created*/
	if( xQueueCAN2 == 0 || xQueueCAN1 == 0 )
	{
		while(1)
		;
	}

	/* Create Tasks */
	periodicSend = xTaskCreate( PeriodicSend, NULL, 512, NULL, SEND_TASK_PRIO, NULL);
	can1MessageReceived = xTaskCreate( CAN1_MessageReceived ,NULL, 512, NULL, REC_TASK_PRIO, NULL );
	can2MessageReceived = xTaskCreate( CAN2_MessageReceived ,NULL, 512, NULL, REC_TASK_PRIO, NULL );

	checkTasks = periodicSend + can1MessageReceived + can2MessageReceived;

	if(checkTasks != numTasks)
	{
		GPIO_SetBits(GPIOD, GPIO_Pin_15);
		GPIO_SetBits(GPIOD, GPIO_Pin_14);
		GPIO_SetBits(GPIOD, GPIO_Pin_13);
		GPIO_SetBits(GPIOD, GPIO_Pin_12);
		while(1)
		{
			for(pauseValue=0; pauseValue<1000000; pauseValue++);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_15);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_14);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_13);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_12);
		}
	}

	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);

	/* Start scheduler */
	/*Ab hier �bernimmt das FreRTOS-Betriebssystem die Kontrolle und die Tasks werden ausgef�hrt*/
	vTaskStartScheduler();

	/* We should never get here as control is now taken by the scheduler */
	for( ;; )
	{
		GPIO_ToggleBits(GPIOD, GPIO_Pin_15);
		vTaskDelayUntil( &xLastWakeTime, 100 );
	}
}



/**
  * @brief  Initializes the LEDs resources.
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14| GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

}




/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
